@@ -82,8 +82,8 @@ class Task(NamedTuple):
 class TaskRunner:
     """The task runner for pdm project"""
 
-    TYPES = ["cmd", "shell", "call", "composite"]
-    OPTIONS = ["env", "env_file", "help", "site_packages"]
+    TYPES = ("cmd", "shell", "call", "composite")
+    OPTIONS = ("env", "env_file", "help", "site_packages")
 
     def __init__(self, project: Project, hooks: HookManager) -> None:
         self.project = project
@@ -248,7 +248,7 @@ class TaskRunner:
                 split = shlex.split(script)
                 cmd = split[0]
                 subargs = split[1:] + ([] if should_interpolate else args)
-                code = self.run(cmd, subargs, options)
+                code = self.run(cmd, subargs, options, chdir=True)
                 if code != 0:
                     return code
             return code
@@ -259,7 +259,7 @@ class TaskRunner:
             **exec_opts(self.global_options, options, opts),
         )
 
-    def run(self, command: str, args: list[str], opts: TaskOptions | None = None) -> int:
+    def run(self, command: str, args: list[str], opts: TaskOptions | None = None, chdir: bool = False) -> int:
         if command in self.hooks.skip:
             return 0
         task = self.get_task(command)
@@ -281,6 +281,7 @@ class TaskRunner:
         else:
             return self._run_process(
                 [command, *args],
+                chdir=chdir,
                 **exec_opts(self.global_options, opts),
             )
 
@@ -335,7 +336,7 @@ class Command(BaseCommand):
     """Run commands or scripts with local packages loaded"""
 
     runner_cls: type[TaskRunner] = TaskRunner
-    arguments = [*BaseCommand.arguments, skip_option, venv_option]
+    arguments = (*BaseCommand.arguments, skip_option, venv_option)
 
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
         action = parser.add_mutually_exclusive_group()
