@@ -19,7 +19,7 @@ from pdm.project import Project
 class Command(BaseCommand):
     """Synchronize the current working set with lock file"""
 
-    arguments = [
+    arguments = (
         *BaseCommand.arguments,
         groups_group,
         dry_run_option,
@@ -28,7 +28,7 @@ class Command(BaseCommand):
         clean_group,
         install_group,
         venv_option,
-    ]
+    )
 
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
         parser.add_argument(
