@@ -30,7 +30,7 @@ if TYPE_CHECKING:
 class Command(BaseCommand):
     """Update package(s) in pyproject.toml"""
 
-    arguments = [
+    arguments = (
         *BaseCommand.arguments,
         groups_group,
         install_group,
@@ -41,7 +41,7 @@ class Command(BaseCommand):
         unconstrained_option,
         skip_option,
         venv_option,
-    ]
+    )
 
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
         parser.add_argument(
@@ -154,24 +154,30 @@ class Command(BaseCommand):
                 for dep in deps.values():
                     dep.specifier = get_specifier("")
         reqs = [r for deps in all_dependencies.values() for r in deps.values()]
-        resolved = do_lock(
-            project,
-            strategy,
-            chain.from_iterable(updated_deps.values()),
-            reqs,
-            dry_run=True,
-            hooks=hooks,
-            groups=locked_groups,
-        )
+        # Since dry run is always true in the locking,
+        # we need to emit the hook manually with the real dry_run value
+        hooks.try_emit("pre_lock", requirements=reqs, dry_run=dry_run)
+        with hooks.skipping("pre_lock", "post_lock"):
+            resolved = do_lock(
+                project,
+                strategy,
+                chain.from_iterable(updated_deps.values()),
+                reqs,
+                dry_run=True,
+                hooks=hooks,
+                groups=locked_groups,
+            )
+        hooks.try_emit("post_lock", resolution=resolved, dry_run=dry_run)
         for deps in updated_deps.values():
             populate_requirement_names(deps)
         if unconstrained:
             # Need to update version constraints
             save_version_specifiers(updated_deps, resolved, save)
-            for group, deps in updated_deps.items():
-                project.add_dependencies(deps, group, selection.dev or False)
         if not dry_run:
-            project.write_lockfile(project.lockfile._data, False, groups=locked_groups)
+            if unconstrained:
+                for group, deps in updated_deps.items():
+                    project.add_dependencies(deps, group, selection.dev or False)
+            project.write_lockfile(project.lockfile._data, False)
         if sync or dry_run:
             do_sync(
                 project,
