@@ -14,7 +14,7 @@ from pdm.project import Project
 class ActivateCommand(BaseCommand):
     """Activate the virtualenv with the given name"""
 
-    arguments = [verbose_option]
+    arguments = (verbose_option,)
 
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
         parser.add_argument("env", nargs="?", help="The key of the virtualenv")
