@@ -4,6 +4,7 @@ import argparse
 import dataclasses as dc
 import json
 import os
+import re
 import sys
 from collections import ChainMap, OrderedDict
 from concurrent.futures import ThreadPoolExecutor
@@ -67,7 +68,9 @@ class PdmFormatter(argparse.RawDescriptionHelpFormatter):
     ) -> str:
         if prefix is None:
             prefix = "Usage: "
-        result = super()._format_usage(usage, actions, groups, prefix)
+        result = super()._format_usage(usage, actions, groups, prefix)  # type: ignore[arg-type]
+        # Remove continuous spaces
+        result = re.sub(r" +", " ", result)
         if prefix:
             return result.replace(prefix, termui.style(prefix, style="warning"))
         return result
@@ -96,6 +99,14 @@ class PdmFormatter(argparse.RawDescriptionHelpFormatter):
             action_header = "%*s%s\n" % tup
             indent_first = help_position
 
+        # Special format for empty action_header
+        # - No extra indent block
+        # - Help info in the same indent level as subactions
+        if not action_header.strip():
+            action_header = ""
+            help_position = self._current_indent
+            indent_first = self._current_indent
+
         # collect the pieces of the action help
         parts = [termui.style(action_header, style="primary")]
 
@@ -109,11 +120,18 @@ class PdmFormatter(argparse.RawDescriptionHelpFormatter):
 
         # or add a newline if the description doesn't end with one
         elif not action_header.endswith("\n"):
-            parts.append("\n")
+            if action_header:
+                parts.append("\n")
 
+        # cancel out extra indent when action_header is empty
+        if not action_header:
+            self._dedent()
         # if there are any sub-actions, add their help as well
         for subaction in self._iter_indented_subactions(action):
             parts.append(self._format_action(subaction))
+        # cancel out extra dedent when action_header is empty
+        if not action_header:
+            self._indent()
 
         # return a single string
         return self._join_parts(parts)
@@ -129,6 +147,7 @@ class ArgumentParser(argparse.ArgumentParser):
         self.add_argument(
             "-h", "--help", action="help", default=argparse.SUPPRESS, help="Show this help message and exit."
         )
+        self._optionals.title = "options"
 
 
 class ErrorArgumentParser(ArgumentParser):
@@ -433,6 +452,9 @@ def format_lockfile(
     project: Project,
     mapping: dict[str, Candidate],
     fetched_dependencies: dict[tuple[str, str | None], list[Requirement]],
+    groups: list[str] | None = None,
+    cross_platform: bool | None = None,
+    static_urls: bool | None = None,
 ) -> dict:
     """Format lock file from a dict of resolved candidates, a mapping of dependencies
     and a collection of package summaries.
@@ -440,30 +462,39 @@ def format_lockfile(
     from pdm.formats.base import make_array, make_inline_table
 
     packages = tomlkit.aot()
-    file_hashes = tomlkit.table()
-    for k, v in sorted(mapping.items()):
+    for _k, v in sorted(mapping.items()):
         base = tomlkit.table()
         base.update(v.as_lockfile_entry(project.root))
         base.add("summary", v.summary or "")
         deps = make_array(sorted(r.as_line() for r in fetched_dependencies[v.dep_key]), True)
         if len(deps) > 0:
             base.add("dependencies", deps)
-        packages.append(base)
         if v.hashes:
-            key = f"{strip_extras(k)[0]} {v.version}"
-            if key in file_hashes:
-                continue
-            array = tomlkit.array().multiline(True)
-            for link, hash_value in sorted(v.hashes.items(), key=lambda l_h: (l_h[0].url_without_fragment, l_h[1])):
-                inline = make_inline_table({"url": link.url_without_fragment, "hash": hash_value})
-                array.append(inline)
-            if array:
-                file_hashes.add(key, array)
+            collected = {}
+            for item in v.hashes:
+                if static_urls:
+                    row = {"url": item["url"], "hash": item["hash"]}
+                else:
+                    row = {"file": item["file"], "hash": item["hash"]}
+                inline = make_inline_table(row)
+                # deduplicate and sort
+                collected[tuple(row.values())] = inline
+            if collected:
+                base.add("files", make_array([collected[k] for k in sorted(collected)], True))
+        packages.append(base)
     doc = tomlkit.document()
-    doc.add("package", packages)
     metadata = tomlkit.table()
-    metadata.add("files", file_hashes)
+    if groups is None:
+        groups = list(project.iter_groups())
+    metadata.update(
+        {
+            "groups": sorted(groups, key=lambda k: k != "default"),
+            "cross_platform": cross_platform if cross_platform is not None else project.lockfile.cross_platform,
+            "static_urls": static_urls if static_urls is not None else project.lockfile.static_urls,
+        }
+    )
     doc.add("metadata", metadata)
+    doc.add("package", packages)
     return cast(dict, doc)
 
 
@@ -601,7 +632,7 @@ def format_resolution_impossible(err: ResolutionImpossible) -> str:
     return "\n".join(result)
 
 
-def merge_dictionary(target: MutableMapping[Any, Any], input: Mapping[Any, Any]) -> None:
+def merge_dictionary(target: MutableMapping[Any, Any], input: Mapping[Any, Any], append_array: bool = True) -> None:
     """Merge the input dict with the target while preserving the existing values
     properly. This will update the target dictionary in place.
     List values will be extended, but only if the value is not already in the list.
@@ -610,10 +641,11 @@ def merge_dictionary(target: MutableMapping[Any, Any], input: Mapping[Any, Any])
         if key not in target:
             target[key] = value
         elif isinstance(value, dict):
-            merge_dictionary(target[key], value)
-        elif isinstance(value, list):
+            merge_dictionary(target[key], value, append_array=append_array)
+        elif isinstance(value, list) and append_array:
             target[key].extend(x for x in value if x not in target[key])
-            target[key].multiline(True)  # type: ignore[attr-defined]
+            if hasattr(target[key], "multiline"):
+                target[key].multiline(True)  # type: ignore[attr-defined]
         else:
             target[key] = value
 
