@@ -17,7 +17,7 @@ class InstallManager:
     """The manager that performs the installation and uninstallation actions."""
 
     # The packages below are needed to load paths and thus should not be cached.
-    NO_CACHE_PACKAGES = ["editables"]
+    NO_CACHE_PACKAGES = ("editables",)
 
     def __init__(self, environment: BaseEnvironment, *, use_install_cache: bool = False) -> None:
         self.environment = environment
