@@ -12,6 +12,7 @@ from typing import TYPE_CHECKING, BinaryIO, Generic, Iterable, TypeVar, cast
 from cachecontrol.cache import SeparateBodyBaseCache
 from cachecontrol.caches import FileCache
 from packaging.utils import canonicalize_name, parse_wheel_filename
+
 from pdm._types import CandidateInfo
 from pdm.exceptions import PdmException
 from pdm.models.candidates import Candidate
@@ -82,6 +83,15 @@ class CandidateInfoCache(JSONFileCache[Candidate, CandidateInfo]):
     candidate -> (dependencies, requires_python, summary) mapping.
     """
 
+    @staticmethod
+    def get_url_part(link: Link) -> str:
+        import base64
+
+        from pdm.utils import url_without_fragments
+
+        url = url_without_fragments(link.split_auth()[1])
+        return base64.urlsafe_b64encode(url.encode()).decode()
+
     @classmethod
     def _get_key(cls, obj: Candidate) -> str:
         # Name and version are set when dependencies are resolved,
@@ -89,7 +99,11 @@ class CandidateInfoCache(JSONFileCache[Candidate, CandidateInfo]):
         if not obj.name or not obj.version:
             raise KeyError("The package is missing a name or version")
         extras = "[{}]".format(",".join(sorted(obj.req.extras))) if obj.req.extras else ""
-        return f"{obj.name}{extras}-{obj.version}"
+        version = obj.version
+        if not obj.req.is_named:
+            assert obj.link is not None
+            version = cls.get_url_part(obj.link)
+        return f"{obj.name}{extras}-{version}"
 
 
 class HashCache:
@@ -102,7 +116,7 @@ class HashCache:
     """
 
     FAVORITE_HASH = "sha256"
-    STRONG_HASHES = ["sha256", "sha384", "sha512"]
+    STRONG_HASHES = ("sha256", "sha384", "sha512")
 
     def __init__(self, directory: Path) -> None:
         self.directory = directory
@@ -128,6 +142,11 @@ class HashCache:
             h.update(chunk)
         return ":".join([h.name, h.hexdigest()])
 
+    def _should_cache(self, link: Link) -> bool:
+        # For now, we only disable caching for local files.
+        # We may add more when we know better about it.
+        return not link.is_file
+
     def get_hash(self, link: Link, session: Session) -> str:
         # If there is no link hash (i.e., md5, sha256, etc.), we don't want
         # to store it.
@@ -142,7 +161,8 @@ class HashCache:
                 hash_value = f"{link.hash_name}:{link.hash}"
             else:
                 hash_value = self._get_file_hash(link, session)
-            self.set(link.url_without_fragment, hash_value)
+            if self._should_cache(link):
+                self.set(link.url_without_fragment, hash_value)
         return hash_value
 
     def _get_path_for_key(self, key: str) -> Path:
