@@ -7,7 +7,7 @@ import re
 import shutil
 import sys
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Iterable, Mapping, cast
+from typing import TYPE_CHECKING, Any, Callable, Iterable, Mapping, cast
 
 import tomlkit
 from tomlkit.items import Array
@@ -379,10 +379,13 @@ class Project:
                 result["pypi"].passive_update(self.default_source)
             merge_sources(self.project_config.iter_sources())
             merge_sources(self.global_config.iter_sources())
+        sources: list[RepositoryConfig] = []
         for source in result.values():
-            assert source.url, "Source URL must not be empty"
+            if not source.url:
+                continue
             source.url = expand_env_vars_in_auth(source.url)
-        return list(result.values())
+            sources.append(source)
+        return sources
 
     def get_repository(
         self, cls: type[BaseRepository] | None = None, ignore_compatibility: bool = True
@@ -473,32 +476,15 @@ class Project:
 
         return SpinnerReporter(spinner, requirements)
 
-    def get_lock_metadata(
-        self, groups: Iterable[str] | None = None, cross_platform: bool | None = None
-    ) -> dict[str, Any]:
-        content_hash = tomlkit.string("sha256:" + self.pyproject.content_hash("sha256"))
-        content_hash.trivia.trail = "\n\n"
-        if groups is None:
-            groups = self.iter_groups()
-        if cross_platform is None:
-            cross_platform = self.lockfile.cross_platform
-        return {
-            "lock_version": self.lockfile.spec_version,
-            "cross_platform": cross_platform,
-            "groups": sorted(groups, key=lambda x: (x != "default", x)),
-            "content_hash": content_hash,
-        }
+    def get_lock_metadata(self) -> dict[str, Any]:
+        content_hash = "sha256:" + self.pyproject.content_hash("sha256")
+        return {"lock_version": self.lockfile.spec_version, "content_hash": content_hash}
 
-    def write_lockfile(
-        self,
-        toml_data: dict,
-        show_message: bool = True,
-        write: bool = True,
-        groups: Iterable[str] | None = None,
-        cross_platform: bool | None = None,
-    ) -> None:
+    def write_lockfile(self, toml_data: dict, show_message: bool = True, write: bool = True, **_kwds: Any) -> None:
         """Write the lock file to disk."""
-        toml_data["metadata"].update(self.get_lock_metadata(groups, cross_platform=cross_platform))
+        if _kwds:
+            deprecation_warning("Extra arguments have been moved to `format_lockfile` function", stacklevel=2)
+        toml_data["metadata"].update(self.get_lock_metadata())
         self.lockfile.set_data(toml_data)
 
         if write:
@@ -524,31 +510,40 @@ class Project:
         content_hash = self.pyproject.content_hash(algo)
         return content_hash == hash_value
 
-    def get_pyproject_dependencies(self, group: str, dev: bool = False) -> tuple[list[str], bool]:
-        """Get the dependencies array in the pyproject.toml
+    def use_pyproject_dependencies(
+        self, group: str, dev: bool = False
+    ) -> tuple[list[str], Callable[[list[str]], None]]:
+        """Get the dependencies array and setter in the pyproject.toml
         Return a tuple of two elements, the first is the dependencies array,
-        and the second tells whether it is a dev-dependencies group.
+        and the second value is a callable to set the dependencies array back.
         """
+
+        def update_dev_dependencies(deps: list[str]) -> None:
+            from tomlkit.container import OutOfOrderTableProxy
+
+            settings.setdefault("dev-dependencies", {})[group] = deps
+            if isinstance(self.pyproject._data["tool"], OutOfOrderTableProxy):
+                # In case of a separate table, we have to remove and re-add it to make the write correct.
+                # This may change the order of tables in the TOML file, but it's the best we can do.
+                # see bug pdm-project/pdm#2056 for details
+                del self.pyproject._data["tool"]["pdm"]
+                self.pyproject._data["tool"]["pdm"] = settings
+
         metadata, settings = self.pyproject.metadata, self.pyproject.settings
         if group == "default":
-            return metadata.setdefault("dependencies", []), False
-        deps_dict = {
-            False: metadata.get("optional-dependencies", {}),
-            True: settings.get("dev-dependencies", {}),
-        }
-        for is_dev, deps in deps_dict.items():
+            return metadata.get("dependencies", tomlkit.array()), lambda x: metadata.__setitem__("dependencies", x)
+        deps_setter = [
+            (
+                metadata.get("optional-dependencies", {}),
+                lambda x: metadata.setdefault("optional-dependencies", {}).__setitem__(group, x),
+            ),
+            (settings.get("dev-dependencies", {}), update_dev_dependencies),
+        ]
+        for deps, setter in deps_setter:
             if group in deps:
-                return deps[group], is_dev
-        if dev:
-            return (
-                settings.setdefault("dev-dependencies", {}).setdefault(group, []),
-                dev,
-            )
-        else:
-            return (
-                metadata.setdefault("optional-dependencies", {}).setdefault(group, []),
-                dev,
-            )
+                return deps[group], setter
+        # If not found, return an empty list and a setter to add the group
+        return tomlkit.array(), deps_setter[int(dev)][1]
 
     def add_dependencies(
         self,
@@ -557,8 +552,7 @@ class Project:
         dev: bool = False,
         show_message: bool = True,
     ) -> None:
-        deps, is_dev = self.get_pyproject_dependencies(to_group, dev)
-        cast(Array, deps).multiline(True)
+        deps, setter = self.use_pyproject_dependencies(to_group, dev)
         for _, dep in requirements.items():
             matched_index = next(
                 (i for i, r in enumerate(deps) if dep.matches(r)),
@@ -569,6 +563,7 @@ class Project:
                 deps.append(req)
             else:
                 deps[matched_index] = req
+        setter(cast(Array, deps).multiline(True))
         self.pyproject.write(show_message)
 
     def init_global_project(self) -> None:
@@ -632,7 +627,7 @@ class Project:
                     yield PythonInfo.from_path(pyenv_shim)
                 elif os.path.exists(pyenv_shim.replace("python3", "python")):
                     yield PythonInfo.from_path(pyenv_shim.replace("python3", "python"))
-            python = shutil.which("python")
+            python = shutil.which("python") or shutil.which("python3")
             if python:
                 yield PythonInfo.from_path(python)
             args = []
@@ -662,18 +657,20 @@ class Project:
 
         from pdm.cli.commands.venv.utils import VenvProvider
 
-        finder = Finder(resolve_symlinks=True)
-        if self.config["python.use_venv"]:
-            finder.add_provider(VenvProvider(self), 0)
+        providers: list[str] = self.config["python.providers"]
+        finder = Finder(resolve_symlinks=True, selected_providers=providers or None)
+        if self.config["python.use_venv"] and (not providers or "venv" in providers):
+            venv_pos = providers.index("venv") if providers else 0
+            finder.add_provider(VenvProvider(self), venv_pos)
         return finder
 
     # compatibility, shouldn't be used directly
     @property
     def meta(self) -> dict[str, Any]:
-        deprecation_warning("project.meta is deprecated, use project.pyproject.metadata instead")
+        deprecation_warning("project.meta is deprecated, use project.pyproject.metadata instead", stacklevel=2)
         return self.pyproject.metadata
 
     @property
     def tool_settings(self) -> dict[str, Any]:
-        deprecation_warning("project.tool_settings is deprecated, use project.pyproject.settings instead")
+        deprecation_warning("project.tool_settings is deprecated, use project.pyproject.settings instead", stacklevel=2)
         return self.pyproject.settings
