@@ -20,7 +20,7 @@ pytest_plugins = [
 """
 from __future__ import annotations
 
-import collections
+import collections.abc
 import json
 import os
 import shutil
@@ -49,12 +49,11 @@ from packaging.version import parse as parse_version
 from pytest_mock import MockerFixture
 from unearth import Link
 
-from pdm.cli.hooks import HookManager
 from pdm.core import Core
 from pdm.environments import BaseEnvironment, PythonEnvironment
 from pdm.exceptions import CandidateInfoNotFound
 from pdm.installers.installers import install_wheel
-from pdm.models.backends import get_backend
+from pdm.models.backends import DEFAULT_BACKEND
 from pdm.models.candidates import Candidate
 from pdm.models.repositories import BaseRepository
 from pdm.models.requirements import (
@@ -72,7 +71,7 @@ if TYPE_CHECKING:
 
     from _pytest.fixtures import SubRequest
 
-    from pdm._types import CandidateInfo, RepositoryConfig
+    from pdm._types import CandidateInfo, FileHash, RepositoryConfig
 
 
 class LocalFileAdapter(requests.adapters.BaseAdapter):
@@ -192,8 +191,8 @@ class TestRepository(BaseRepository):
             self._get_dependencies_from_metadata,
         )
 
-    def get_hashes(self, candidate: Candidate) -> dict[Link, str] | None:
-        return {}
+    def get_hashes(self, candidate: Candidate) -> list[FileHash]:
+        return []
 
     def _find_candidates(self, requirement: Requirement) -> Iterable[Candidate]:
         for version, candidate in sorted(
@@ -419,16 +418,23 @@ def project(project_no_init: Project) -> Project:
     Returns:
         The initialized project
     """
-    from pdm.cli.commands.init import Command
+    from pdm.cli.utils import merge_dictionary
 
-    hooks = HookManager(project_no_init, ["post_init"])
-    Command.do_init(
-        project_no_init,
-        "test_project",
-        "0.0.0",
-        hooks=hooks,
-        build_backend=get_backend("pdm-pep517"),
-    )
+    data = {
+        "project": {
+            "name": "test-project",
+            "version": "0.0.0",
+            "description": "",
+            "authors": [],
+            "license": {"text": "MIT"},
+            "dependencies": [],
+            "requires-python": ">=3.7",
+        },
+        "build-system": DEFAULT_BACKEND.build_system(),
+    }
+
+    merge_dictionary(project_no_init.pyproject._data, data)
+    project_no_init.pyproject.write()
     # Clean the cached property
     project_no_init._environment = None
     return project_no_init
@@ -583,7 +589,7 @@ if TYPE_CHECKING:
 @pytest.fixture
 def pdm(core: Core, monkeypatch: pytest.MonkeyPatch) -> PDMCallable:
     """
-    A fixture alloowing to execute PDM commands
+    A fixture allowing to execute PDM commands
 
     Returns:
         A `pdm` fixture command.
