@@ -50,11 +50,12 @@ class BaseProvider(AbstractProvider):
         """
         editable = requirement.editable
         is_named = requirement.is_named
+        is_pinned = requirement.is_pinned
         is_prerelease = (
             requirement.prerelease or requirement.specifier is not None and bool(requirement.specifier.prereleases)
         )
         specifier_parts = len(requirement.specifier) if requirement.specifier else 0
-        return (not editable, is_named, not is_prerelease, -specifier_parts)
+        return (not editable, is_named, not is_pinned, not is_prerelease, -specifier_parts)
 
     def identify(self, requirement_or_candidate: Requirement | Candidate) -> str:
         return requirement_or_candidate.identify()
@@ -165,7 +166,10 @@ class BaseProvider(AbstractProvider):
         if not requirement.is_named:
             if candidate.req.is_named:
                 return False
-            return self._compare_file_reqs(requirement, candidate.req)  # type: ignore[arg-type]
+            can_req = candidate.req
+            if requirement.is_vcs and can_req.is_vcs:
+                return can_req.vcs == requirement.vcs and can_req.repo == requirement.repo  # type: ignore[attr-defined]
+            return self._compare_file_reqs(requirement, can_req)  # type: ignore[arg-type]
         version = candidate.version
         this_name = self.repository.environment.project.name
         if version is None or candidate.name == this_name:
